/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8130_SUPSTPS.sql

Descrio:
Calcula o custo da lista de preo inflacionada

Criao:
07/01/2005 Joo Fernando dos Reis Corra
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'CUSTO_LISTAINFLAC'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
   DROP PROCEDURE CUSTO_LISTAINFLAC;
GO

CREATE PROCEDURE CUSTO_LISTAINFLAC
   @P_CDFILCENTCMP VARCHAR(4),
   @P_NRLISTINFLAC VARCHAR(10),
   @P_CDOPERADOR   VARCHAR(12)
AS
   DECLARE @V_DTREFLISTINF DATETIME;
   DECLARE @V_DTINILISTINF DATETIME;
   DECLARE @V_DTFINLISTINF DATETIME;

   DECLARE @V_DTFINCUSTOLP DATETIME;

   DECLARE @V_DTAUXREF     DATETIME;

   DECLARE @C_PERLISINFLAC_CDFILIAL     VARCHAR(4);
   DECLARE @C_PERLISINFLAC_CDPRODUTO    VARCHAR(10);
   DECLARE @C_PERLISINFLAC_VRPERLISTINF DOUBLE PRECISION;
   DECLARE @C_PERLISINFLAC_VRCUSPRLISTP DOUBLE PRECISION;
   DECLARE @C_PERLISINFLAC_VRCUSPRLISM1 DOUBLE PRECISION;
   DECLARE @C_PERLISINFLAC_VRCUSPRLISM2 DOUBLE PRECISION;
   DECLARE @C_PERLISINFLAC_DTCUSTOSUP   DATETIME;
   DECLARE @C_PERLISINFLAC_DTFINCUSTSUP DATETIME;
BEGIN
   SELECT
      @V_DTREFLISTINF = DTREFLISTINF,
      @V_DTINILISTINF = DTINILISTINF,
      @V_DTFINLISTINF = DTFINLISTINF
   FROM
      LISTAINFLAC
   WHERE
      CDFILCENTCMP = @P_CDFILCENTCMP AND
      NRLISTINFLAC = @P_NRLISTINFLAC;

   INSERT INTO LOGLISINFLAC
      (CDFILCENTLLI, NRLISTINFLLI, CDOPERADOR, DTGERLISINF)
   VALUES
      (@P_CDFILCENTCMP, @P_NRLISTINFLAC, @P_CDOPERADOR, GETDATE());

   DECLARE C_PERLISINFLAC CURSOR FOR
      SELECT
         FILIAL.CDFILIAL,
         A.CDPRODUTO,
         PERLISINFLAC.VRPERLISTINF,
         CUSTOSUP.VRCUSPRLISTP,
         CUSTOSUP.VRCUSPRLISM1,
         CUSTOSUP.VRCUSPRLISM2,
         CUSTOSUP.DTCUSTOSUP,
         CUSTOSUP.DTFINCUSTSUP
      FROM
         CUSTOSUP,
         PRODUTO,
         (SELECT
             P.CDPRODUTO,
             MAX(PRODUTO.CDARVPROD) AS CDPRODNIVEL
          FROM
             PRODUTO,
             PRODUTO P,
             PERLISINFLAC
          WHERE
             PERLISINFLAC.CDFILCENTCMP = @P_CDFILCENTCMP AND
             PERLISINFLAC.NRLISTINFLAC = @P_NRLISTINFLAC AND
             PERLISINFLAC.CDPRODUTO = PRODUTO.CDPRODUTO AND
             P.CDARVPROD LIKE PRODUTO.CDARVPROD + '%' AND
             P.CDPRODESTO IS NOT NULL
         GROUP BY
            P.CDPRODUTO) A,
         FILIAL,
         PERLISINFLAC
      WHERE
         PERLISINFLAC.CDFILCENTCMP = @P_CDFILCENTCMP AND
         PERLISINFLAC.NRLISTINFLAC = @P_NRLISTINFLAC AND
         ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) = PERLISINFLAC.CDFILCENTCMP AND
         PERLISINFLAC.CDPRODUTO = PRODUTO.CDPRODUTO AND
         PRODUTO.CDARVPROD = A.CDPRODNIVEL AND
         (ISNULL(CUSTOSUP.VRCUSPRLISTP, 0) + ISNULL(CUSTOSUP.VRCUSPRLISM1, 0) + ISNULL(CUSTOSUP.VRCUSPRLISM2, 0)) > 0 AND
         CUSTOSUP.IDCUSLISTINF <> 'S' AND
         CUSTOSUP.CDFILIAL = FILIAL.CDFILIAL AND
         CUSTOSUP.CDPRODUTO = A.CDPRODUTO AND
         CUSTOSUP.DTCUSTOSUP = (SELECT
                                   MAX(C.DTCUSTOSUP)
                                FROM
                                   CUSTOSUP C
                                WHERE
                                   C.CDFILIAL = CUSTOSUP.CDFILIAL AND
                                   C.CDPRODUTO = CUSTOSUP.CDPRODUTO AND
                                   C.DTCUSTOSUP <= @V_DTREFLISTINF AND
                                  (ISNULL(C.VRCUSPRLISTP, 0) + ISNULL(C.VRCUSPRLISM1, 0) + ISNULL(C.VRCUSPRLISM2, 0)) > 0 AND
                                   C.IDCUSLISTINF <> 'S');
   OPEN C_PERLISINFLAC;
   FETCH NEXT FROM C_PERLISINFLAC INTO @C_PERLISINFLAC_CDFILIAL,
                                       @C_PERLISINFLAC_CDPRODUTO,
                                       @C_PERLISINFLAC_VRPERLISTINF,
                                       @C_PERLISINFLAC_VRCUSPRLISTP,
                                       @C_PERLISINFLAC_VRCUSPRLISM1,
                                       @C_PERLISINFLAC_VRCUSPRLISM2,
                                       @C_PERLISINFLAC_DTCUSTOSUP,
                                       @C_PERLISINFLAC_DTFINCUSTSUP;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      SELECT       /*PARA NAO INCLUIR CUSTO DE LISTA INFLACIONADA SE JA HOVER CUSTO DE LISTA DE PRECO*/
         @V_DTFINCUSTOLP = MAX(C.DTFINCUSTSUP)
      FROM
         CUSTOSUP C
      WHERE
         C.CDFILIAL = @C_PERLISINFLAC_CDFILIAL AND
         C.CDPRODUTO = @C_PERLISINFLAC_CDPRODUTO AND
         C.DTCUSTOSUP <= @V_DTFINLISTINF AND
         C.DTFINCUSTSUP >= @V_DTINILISTINF AND
         (ISNULL(C.VRCUSPRLISTP, 0) + ISNULL(C.VRCUSPRLISM1, 0) + ISNULL(C.VRCUSPRLISM2, 0)) > 0 AND
         C.IDCUSLISTINF <> 'S';

      SET @V_DTAUXREF = @V_DTINILISTINF; /*A LISTA INFLACIONADA NO PODE SOBREPOR UMA LISTA DE PREO, NESTE CASO ELA  ADIANTADA.*/
      IF @V_DTFINCUSTOLP >= @V_DTINILISTINF BEGIN
        SET @V_DTAUXREF = @V_DTFINCUSTOLP + 1;
      END;
      IF @V_DTAUXREF <= @V_DTFINLISTINF BEGIN
         DELETE FROM CUSTOSUP
         WHERE
            CDFILIAL = @C_PERLISINFLAC_CDFILIAL AND
            CDPRODUTO = @C_PERLISINFLAC_CDPRODUTO AND
            DTCUSTOSUP = @V_DTAUXREF;
         INSERT INTO CUSTOSUP (CDFILIAL,     CDPRODUTO,    DTCUSTOSUP,   VRCUSPRLISTP,
                               VRCUSPRLISM1, VRCUSPRLISM2, DTFINCUSTSUP, CDFILCENTCMP,
                               NRLISTINFLAC, VRPERLISINFL, CDFILIALORIG, CDPRODORIG,
                               DTCUSTSUPORI, IDCUSLISTINF)
         VALUES (@C_PERLISINFLAC_CDFILIAL,
                 @C_PERLISINFLAC_CDPRODUTO,
                 @V_DTAUXREF,
                 @C_PERLISINFLAC_VRCUSPRLISTP*(1 + (@C_PERLISINFLAC_VRPERLISTINF / 100)),
                 @C_PERLISINFLAC_VRCUSPRLISM1*(1 + (@C_PERLISINFLAC_VRPERLISTINF / 100)),
                 @C_PERLISINFLAC_VRCUSPRLISM2*(1 + (@C_PERLISINFLAC_VRPERLISTINF / 100)),
                 @V_DTFINLISTINF,
                 @P_CDFILCENTCMP,
                 @P_NRLISTINFLAC,
                 @C_PERLISINFLAC_VRPERLISTINF,
                 @C_PERLISINFLAC_CDFILIAL,
                 @C_PERLISINFLAC_CDPRODUTO,
                 @C_PERLISINFLAC_DTCUSTOSUP,
                 'S');
      END;
      FETCH NEXT FROM C_PERLISINFLAC INTO @C_PERLISINFLAC_CDFILIAL,
                                          @C_PERLISINFLAC_CDPRODUTO,
                                          @C_PERLISINFLAC_VRPERLISTINF,
                                          @C_PERLISINFLAC_VRCUSPRLISTP,
                                          @C_PERLISINFLAC_VRCUSPRLISM1,
                                          @C_PERLISINFLAC_VRCUSPRLISM2,
                                          @C_PERLISINFLAC_DTCUSTOSUP,
                                          @C_PERLISINFLAC_DTFINCUSTSUP;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'CUSTO_LISTAINFLAC','5.01.018';
GO

